package com.incubator.common.net.netty;

import com.incubator.common.log4j.Log;
import com.incubator.common.thread.NamedThreadFactory;
import io.netty.bootstrap.Bootstrap;
import io.netty.buffer.PooledByteBufAllocator;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.pool.*;
import io.netty.channel.socket.nio.NioSocketChannel;

import java.net.InetSocketAddress;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 高性能优化版 NettyClient
 */
public final class NettyClient {

	private static final int DEFAULT_IO_THREADS = Math.max(1, Runtime.getRuntime().availableProcessors() * 2);
	private static final int CONNECT_TIMEOUT_MILLIS = 5000; // 默认连接超时时间
	private static final int BASE_RECONNECT_INTERVAL = 1000; // 初始重连间隔
	private static final int MAX_RECONNECT_INTERVAL = 60000; // 最大重连间隔

	private final InetSocketAddress target;
	private final EventLoopGroup ioWorkers;
	private final Bootstrap bootstrap;
	private final ChannelPool channelPool;

	private volatile boolean reconnectEnabled = false;
	private volatile long currentReconnectInterval = BASE_RECONNECT_INTERVAL;
	private ScheduledExecutorService reconnectExecutor;

	public NettyClient(String host, int port, ChannelHandler handler) {
		this(new InetSocketAddress(host, port), handler, DEFAULT_IO_THREADS);
	}

	public NettyClient(InetSocketAddress target, ChannelHandler handler, int ioThreads) {
		this.target = target;

		// 初始化 IO 线程组
		this.ioWorkers = new NioEventLoopGroup(ioThreads, new NamedThreadFactory("Client-ioWorkers", true));

		// 初始化 Bootstrap
		this.bootstrap = new Bootstrap()
				.group(ioWorkers)
				.channel(NioSocketChannel.class)
				.option(ChannelOption.ALLOCATOR, PooledByteBufAllocator.DEFAULT) // 使用内存池
				.option(ChannelOption.CONNECT_TIMEOUT_MILLIS, CONNECT_TIMEOUT_MILLIS)
				.option(ChannelOption.TCP_NODELAY, true)
				.option(ChannelOption.SO_KEEPALIVE, true)
				.handler(new ChannelInitializer<Channel>() {
					@Override
					protected void initChannel(Channel ch) {
						ch.pipeline().addLast(handler);
					}
				});

		// 初始化连接池
		this.channelPool = createChannelPool();
	}

	/**
	 * 创建连接池
	 */
	private ChannelPool createChannelPool() {
		return new SimpleChannelPool(bootstrap, new CustomChannelPoolHandler());
	}

	/**
	 * 启动客户端并连接目标服务器
	 */
	public void start() {
		connect();
	}

	/**
	 * 停止客户端并释放资源
	 */
	public void stop() {
		try {
			channelPool.close();
			ioWorkers.shutdownGracefully().sync();
			Log.info("客户端已停止");
		} catch (InterruptedException e) {
			Thread.currentThread().interrupt();
			Log.error("客户端停止时发生异常: {}", e.getMessage(), e);
		}
	}

	/**
	 * 启用断线重连
	 */
	public void enableReconnect(ScheduledExecutorService executor) {
		if (executor == null) {
			throw new IllegalArgumentException("重连调度器不能为空");
		}
		this.reconnectEnabled = true;
		this.reconnectExecutor = executor;
		Log.info("断线重连已启用");
	}

	/**
	 * 禁用断线重连
	 */
	public void disableReconnect() {
		this.reconnectEnabled = false;
		Log.info("断线重连已禁用");
	}

	/**
	 * 发送消息
	 */
	public void sendMessage(Object message) {
		channelPool.acquire().addListener((future) -> {
			if (future.isSuccess()) {
				Channel channel = (Channel) future.getNow();
				channel.writeAndFlush(message).addListener(f -> {
					if (f.isSuccess()) {
						Log.info("消息发送成功: {}", message);
					} else {
						Log.error("消息发送失败: {}", f.cause());
					}
				});
				channelPool.release(channel); // 释放通道
			} else {
				Log.error("获取通道失败: {}", future.cause());
			}
		});
	}

	/**
	 * 执行连接
	 */
	private void connect() {
		channelPool.acquire().addListener((future) -> {
			if (future.isSuccess()) {
				Log.info("连接成功: {}", target);
				resetReconnectInterval(); // 重置重连间隔
			} else {
				Log.warn("连接失败: {}", target);
				if (reconnectEnabled) {
					scheduleReconnect();
				}
			}
		});
	}

	/**
	 * 调度重连
	 */
	private void scheduleReconnect() {
		if (reconnectEnabled && reconnectExecutor != null) {
			reconnectExecutor.schedule(this::connect, currentReconnectInterval, TimeUnit.MILLISECONDS);
			Log.info("将在 {} 毫秒后尝试重连", currentReconnectInterval);
			increaseReconnectInterval();
		}
	}

	/**
	 * 增加重连间隔（指数回退）
	 */
	private void increaseReconnectInterval() {
		currentReconnectInterval = Math.min(currentReconnectInterval * 2, MAX_RECONNECT_INTERVAL);
	}

	/**
	 * 重置重连间隔
	 */
	private void resetReconnectInterval() {
		currentReconnectInterval = BASE_RECONNECT_INTERVAL;
	}

	/**
	 * 自定义 ChannelPoolHandler
	 */
	private static class CustomChannelPoolHandler implements ChannelPoolHandler {
		@Override
		public void channelReleased(Channel ch) {
			Log.info("通道已释放: {}", ch);
		}

		@Override
		public void channelAcquired(Channel ch) {
			Log.info("通道已获取: {}", ch);
		}

		@Override
		public void channelCreated(Channel ch) {
			Log.info("新通道已创建: {}", ch);
		}
	}
}
