package com.incubator.common.net.netty;

import java.net.InetSocketAddress;
import java.util.concurrent.Executors;

import com.incubator.common.log4j.Log;
import com.incubator.common.thread.NamedThreadFactory;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.buffer.PooledByteBufAllocator;
import io.netty.channel.*;
import io.netty.channel.epoll.Epoll;
import io.netty.channel.epoll.EpollServerSocketChannel;
import io.netty.channel.group.ChannelGroup;
import io.netty.channel.group.DefaultChannelGroup;
import io.netty.channel.kqueue.KQueue;
import io.netty.channel.kqueue.KQueueServerSocketChannel;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.ServerSocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.util.concurrent.ImmediateEventExecutor;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * 高并发优化版 NettyServer
 */
public final class NettyServer {

	private static final int DEFAULT_BOSS_THREADS = Math.max(1, Runtime.getRuntime().availableProcessors() / 2);
	private static final int DEFAULT_IO_THREADS = Math.max(1, Runtime.getRuntime().availableProcessors() * 2);

	private final InetSocketAddress local;
	private final ServerBootstrap bootstrap = new ServerBootstrap();
	private final EventLoopGroup bossGroup;
	private final EventLoopGroup workerGroup;
	private final ChannelGroup channelGroup = new DefaultChannelGroup("NettyServerChannelGroup", ImmediateEventExecutor.INSTANCE);
	private final AtomicBoolean isRunning = new AtomicBoolean(false);
	private final ExecutorService businessExecutor = Executors.newFixedThreadPool(
			Runtime.getRuntime().availableProcessors() * 2,
			new NamedThreadFactory("BusinessExecutor", true)
	);

	private String name = "NettyServer";
	private ServerSocketChannel serverChannel;

	public NettyServer(String host, int port) {
		this.local = new InetSocketAddress(host, port);
		this.bossGroup = new NioEventLoopGroup(DEFAULT_BOSS_THREADS, new NamedThreadFactory("Server-boss", true));
		this.workerGroup = new NioEventLoopGroup(DEFAULT_IO_THREADS, new NamedThreadFactory("Server-ioWorkers", true));
		initBootstrap();
	}

	private void initBootstrap() {
		bootstrap.group(bossGroup, workerGroup)
				.channel(getDefaultChannelClass())
				.option(ChannelOption.ALLOCATOR, PooledByteBufAllocator.DEFAULT)
				.option(ChannelOption.SO_BACKLOG, 1024) // 提升连接等待队列大小
				.option(ChannelOption.SO_REUSEADDR, true) // 重用地址
				.childOption(ChannelOption.ALLOCATOR, PooledByteBufAllocator.DEFAULT)
				.childOption(ChannelOption.TCP_NODELAY, true) // 关闭 Nagle 算法，提升小数据传输性能
				.childOption(ChannelOption.SO_KEEPALIVE, true) // 保持连接
				.childOption(ChannelOption.SO_RCVBUF, 64 * 1024) // 增大接收缓冲区大小
				.childOption(ChannelOption.SO_SNDBUF, 64 * 1024) // 增大发送缓冲区大小
				.childOption(ChannelOption.WRITE_BUFFER_WATER_MARK, new WriteBufferWaterMark(32 * 1024, 64 * 1024)); // 提升写缓冲区大小
	}

	public boolean start(ChannelHandler handler) {
		if (!isRunning.compareAndSet(false, true)) {
			Log.warn("[{}] 服务已经在运行中", name);
			return false;
		}
		try {
			bootstrap.childHandler(new ChannelInitializer<Channel>() {
				@Override
				protected void initChannel(Channel ch) {
					channelGroup.add(ch);
					ch.pipeline().addLast(handler);
				}
			});

			ChannelFuture future = bootstrap.bind(local).sync();
			serverChannel = (ServerSocketChannel) future.channel();

			if (future.isSuccess()) {
				Log.info("[{}] 服务启动成功，绑定端口: {}", name, local);
				return true;
			} else {
				Log.error("[{}] 服务启动失败: {}", name, future.cause());
				return false;
			}
		} catch (Exception e) {
			Log.error("[{}] 服务启动异常: {}", name, e.getMessage(), e);
			stop();
			return false;
		}
	}

	public void stop() {
		if (!isRunning.compareAndSet(true, false)) {
			Log.warn("[{}] 服务已经停止", name);
			return;
		}
		try {
			if (serverChannel != null) {
				serverChannel.close().sync();
			}
			channelGroup.close();
			shutdownEventLoopGroup(bossGroup, "BossGroup");
			shutdownEventLoopGroup(workerGroup, "WorkerGroup");
			businessExecutor.shutdownNow();
			Log.info("[{}] 服务停止成功", name);
		} catch (Exception e) {
			Log.error("[{}] 服务停止异常: {}", name, e.getMessage(), e);
		}
	}

	public void broadCast(Object message) {
		channelGroup.writeAndFlush(message);
		Log.debug("[{}] 广播消息，总连接数: {}", name, channelGroup.size());
	}

	public int getConnectionCount() {
		return channelGroup.size();
	}

	public boolean isActive() {
		return serverChannel != null && serverChannel.isActive();
	}

	public void setName(String name) {
		this.name = name;
	}

	private Class<? extends ServerChannel> getDefaultChannelClass() {
		if (Epoll.isAvailable()) {
			return EpollServerSocketChannel.class;
		} else if (KQueue.isAvailable()) {
			return KQueueServerSocketChannel.class;
		} else {
			return NioServerSocketChannel.class;
		}
	}

	private void shutdownEventLoopGroup(EventLoopGroup group, String groupName) {
		if (group != null) {
			try {
				group.shutdownGracefully().sync();
				Log.info("[{}] {} 已关闭", name, groupName);
			} catch (InterruptedException e) {
				Thread.currentThread().interrupt();
				Log.error("[{}] {} 关闭失败: {}", name, groupName, e.getMessage(), e);
			}
		}
	}
}
