package com.incubator.game.contest;

import com.alibaba.fastjson2.JSONObject;
import com.incubator.common.log4j.Log;
import com.incubator.common.util.DateUtils;
import com.incubator.common.util.StringUtils;
import com.incubator.game.data.data.GameData;
import com.incubator.game.data.data.PromotionLvCO;
import com.incubator.game.data.data.ext.ContestExt;
import com.incubator.game.data.po.ContestPO;
import com.incubator.game.player.Player;
import com.incubator.game.room.JDGDRoom;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.PlayerUtil;
import com.incubator.game.util.ProtoUtil;
import com.incubator.game.util.Struct;
import com.incubator.message.proto.CommonProto;

import java.util.*;
import java.util.concurrent.*;

/**
 * 官方比赛实体类
 */
public class Contest {

    /** 比赛数据 **/
    public ContestPO data;
    /** 比赛策划表配置 **/
    public ContestExt prop;
    /** 当前晋级配置 **/
    public PromotionLvCO propPromotionLv;

    /** 激活标记 **/
    private boolean isActive;
    /** 检测标记 **/
    public boolean flag;
    /** 时间秒数 **/
    public int time;
    /** 比赛状态(0等待/报名中 1比赛开始 2进行中 3已结束) **/
    public int state;

    /** 每个房间玩家容量 **/
    private final int roomCapacity = 2;

    /** 比赛报名角色 [key:玩家id, value:玩家对象]**/
    public Map<String, Player> actors = new ConcurrentHashMap<>();
    /** 比赛房间列表 **/
    public Map<Long, Room> rooms = new ConcurrentHashMap<>();
    /** 等待队列 **/
    private final ConcurrentLinkedQueue<String> waitingPlayers = new ConcurrentLinkedQueue<>();
    /** 是否进行下一轮 **/
    public boolean isNext = true;

    /** 比赛任务主线程 **/
    private ScheduledExecutorService scheduler;

    public Contest() {
        if (this.scheduler == null || this.scheduler.isShutdown()) {
            this.scheduler = Executors.newSingleThreadScheduledExecutor();
        }
    }

    /**
     * 初始化
     *
     * @param roomType  比赛玩法类型
     * @param contestId 比赛id
     * @param prop      比赛策划表配置
     */
    public void init(int roomType, long contestId, ContestExt prop) {
        this.prop = prop;
        // 初始化默认第一条配置
        this.propPromotionLv = GameData.PromotionLvs.get(1);
        // 初始化比赛
        this.data = new ContestPO();
        this.data.contestId = contestId;
        // todo 测试
//        this.data.startTime = prop.startTime;
        this.data.startTime = System.currentTimeMillis() + (2 * 60 * 1000);
        this.data.contestType = 1;
        this.data.level = 1;
    }

    /**
     * 根据当前人数刷新晋级配置
     *
     * @param num   实际报名人数
     */
    public void refreshPromotionLv(int num) {
        for (PromotionLvCO config : GameData.PromotionLvs.values()) {
            if (num >= config.min && num <= config.max) {
                this.propPromotionLv = config;
            }
        }
    }

    /**
     * 玩家报名加入比赛
     *
     * @param player
     */
    public synchronized void joinContest(Player player) {
        // 加入比赛角色缓存
        if (!this.actors.containsKey(player.getId())) {
            this.actors.put(player.getId(), player);
        }
        if (!this.data.playerList.contains(player.getId())) {
            this.data.playerList.add(player.getId());
        }
        // 加入等待队列
        if (!this.waitingPlayers.contains(player.getId())) {
            this.waitingPlayers.add(player.getId());
        }
        // 根据报名人数,刷新晋级配置
        refreshPromotionLv(this.actors.size());
        if (this.propPromotionLv == null) {
            Log.error("根据报名人数刷新晋级配置出错...比赛 : {}, {}, 人数 : {}",
                    this.data.contestId, this.prop.desc, this.actors.size());
        }
    }

    /**
     * 开启官方比赛主线程
     */
    public void startContestTask() {
        this.isActive = true;
        this.scheduler.scheduleAtFixedRate(() -> {
            if (this.isActive) {
                this.doUpdate();
            }
        }, 0, 1, TimeUnit.SECONDS);
    }

    /**
     * 比赛主线程逻辑
     */
    public void doUpdate() {
//        Log.debug("检测: 官方比赛主线程执行...");
        switch (this.state) {
            case 0:
//                Log.debug("检测: 官方比赛主线程执行...等待开始...比赛id : {}, {}, 人数 : {}, 比赛开始时间 : {}",
//                        this.data.contestId, this.prop.desc, this.actors.size(), DateUtils.formatFullDate(new Date(this.data.startTime)));
                // 等待开始
                if (this.checkReadyStart()) {
                    this.state = 1;
                    this.flag = false;
                    this.time = 0;
                }
                break;
            case 1:
                // 比赛开始
                if (!this.flag) {
                    //开始
                    if (this.time >= 3) {
                        this.doStart();
                        this.state = 2;
                        this.flag = false;
                        this.time = 0;
                        Log.debug("检测: 比赛id:{}, 比赛人数:{}, 状态:比赛开始...", this.data.contestId, this.actors.size());
                    }
                }
                break;
            case 2:
                // 进行中
                if (!this.flag) {
                    // 检测当轮是否结束
                    this.checkRoomsAllOver();
                } else {

                }
                break;
            case 3:
                // 结束
                if (!this.flag) {
                    this.flag = true;
                    this.time = 0;
                    Log.debug("比赛结束: 比赛id : {}, 比赛人数 : {}, 当前轮次 : {}",
                            this.data.contestId, this.actors.size(), this.data.curRound);
                } else {
                    // 可以下一轮,切等待状态
                    if (this.isNext) {
                        this.resetContest();
                        this.state = 0;
                        this.flag = false;
                        this.time = 0;
                    } else {
                        // 120秒未开始直接解散
//                        if (this.time >= 120) {
//                            // 广播
//                            for (Player tmPlayer : this.data.playerMap.values()) {
//                                if (tmPlayer != null) {
//                                    tmPlayer.receive(CommonProto.Cmd.DisbandRoom_VALUE, new HashMap<>());
//                                }
//                            }
//                            this.destroy();
//                        }
                    }
                }
                break;
        }

        this.time += 1;
    }

    /**
     * 检测是否可以开始 默认3秒后比赛开始
     *
     * @return
     */
    public boolean checkReadyStart() {
        Date now = new Date();
        if (!this.isStart(now) || this.state != 0) {
            return false;
        }

        // 广播3秒倒计时
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = ProtoUtil.contestInfoToMessage(this);
                data.put("contestState", 1);
                data.put("contestTime", this.data.startTime);
                Log.debug("比赛准备开始 : 比赛 : {}, 开始时间 : {}, 人数 : {}", this.data.contestId, DateUtils.formatFullDate(new Date(this.data.startTime)), this.actors.size());
                tmPlayer.receive(CommonProto.Cmd.MatchPreparationStart_VALUE, data);
            }
        }

        return true;
    }

    /**
     * 比赛是否可开始
     *
     * @return
     */
    public boolean isStart(Date date) {
        // 时间是否到了
        return date.getTime() >= this.data.startTime - (3 * 1000);
    }

    /**
     * 根据玩家实时人数，动态分配比赛桌子
     */
    public synchronized void allocateRooms() {
        while (this.waitingPlayers.size() >= roomCapacity) {
            // 玩家匹配,分配房间
            JSONObject jsonData = new JSONObject();
            jsonData.put("playingTypeOne", 2);
            jsonData.put("playingTypeTwo", 1);
            jsonData.put("upgradeType", 1);
            jsonData.put("flippingType", 2);
            jsonData.put("advanceType", 2);
            jsonData.put("timeoutType", 1);
            jsonData.put("voiceType", 2);
            jsonData.put("recordType", 2);
            jsonData.put("watchType", 4);
            // 房间id
            jsonData.put("roomId", RoomService.getInstance().randomRoomIdByContest());
            // 人数模式 1:2人 2:4人
            jsonData.put("PlayerNum", 2);
            // 房间最大局数
            jsonData.put("maxRound", this.prop.maxRound);
            // 房间类型 0:好友房 1:官方比赛房
            jsonData.put("type", 1);
            // 房间玩法类型 1:掼蛋
            jsonData.put("gameType", 1);

            Room room = null;
            try {
                room = RoomService.getInstance().creatRoom(null, 1, jsonData);
            } catch (Exception e) {
                Log.error("比赛开始...创建房间出错...", e);
            }
            if (room == null) {
                Log.error("比赛开始...创建房间出错...比赛 : {}, name : {}", this.data.contestId, this.prop.desc);
                return;
            }

            // 开启房间主线程
            room.startRoomTask();

            // 添加房间缓存
            this.rooms.put(room.roomId, room);

            Log.info("比赛开始...比赛 : {}, name : {}, Room created: id={}, data={}",
                    this.data.contestId, this.prop.desc, room.roomId, room.jsonData);

            // 推送广播,拉玩家进房间
            for (int i = 0; i < this.roomCapacity; i++) {
                String playerId = this.waitingPlayers.poll();
                if (!StringUtils.isEmpty(playerId)) {
                    Player tmPlayer = (Player) PlayerUtil.getOnlinePlayer(playerId);
                    if (tmPlayer == null) {
                        // todo 此处首轮需要替补机器人
                        continue;
                    }

//                    if (!list.contains(tmPlayer.getValue().getId())) {
//                        list.add(tmPlayer.getValue().getId());
//                    }
//                    tmpRooms.put(room.roomId, list);

                    Map<String, Object> data = ProtoUtil.contestInfoToMessage(this);
                    data.put("roomId", room.roomId);
                    data.put("roomType", room.type);
                    Log.debug("比赛开始 : 比赛 : {}, 玩家 : {}", this.data.contestId, this.actors);
                    tmPlayer.receive(CommonProto.Cmd.MatchOfficialStart_VALUE, data);
                }
            }
        }

        // 比赛房间缓存列表
//        Map<Long, List<String>> tmpRooms = this.data.rooms.getOrDefault(this.data.curRound, new HashMap<>());
//        this.data.rooms.put(this.data.curRound, tmpRooms);
    }

    /**
     * 比赛开始
     */
    public void doStart() {
        // 检测比赛人数是否满足最小人数配置
//        if (this.actors.size() < this.propPromotionLv.min) {
//            // 流局结束
//            this.gameOver();
//            return;
//        }
        // 设置比赛数据
        this.data.curRound += 1;
        // 动态分配比赛房间
        this.allocateRooms();
    }

    /**
     * 检测桌子是否全部结束
     */
    public void checkRoomsAllOver() {
        int cnt = 0;
        for (Room room : this.rooms.values()) {
            if (room == null) {
                continue;
            }
            switch (room.gameType) {
                case 1:
                    if (((JDGDRoom) room).data.curRound >= this.prop.maxRound && room.state == 3) {
                        cnt += 1;
                    }
                    break;
            }
        }
        if (cnt == this.rooms.size()) {
            // 晋级
            this.promotionLv();
            // 结算
            this.settlement();
            // 结束
            this.gameOver();
        }
    }

    /**
     * 晋级
     */
    private void promotionLv() {
        Map<Long, List<Struct.PromotionData>> roomMap = this.data.promotionMap.getOrDefault(this.data.curRound, new HashMap<>());
        // 每个房间需要晋级的人数
        int cnt;

        // 首轮按配置表晋级,后面按头游二游晋级
        if (this.data.curRound == 1) {
            cnt = Math.floorDiv(this.propPromotionLv.num, this.rooms.size());
        } else {
            cnt = this.prop.model == 4? 2 : 1;
        }

        for (Room room : this.rooms.values()) {
            if (room == null) {
                continue;
            }
            List<Struct.PromotionData> list = roomMap.getOrDefault(room.roomId, new ArrayList<>());
            switch (room.gameType) {
                case 1:
                    if (((JDGDRoom) room).data.result.size() >= ((JDGDRoom) room).data.maxNum) {
                        for (int i = 1; i < ((JDGDRoom) room).data.result.size(); i++) {
                            Player player = ((JDGDRoom) room).data.result.getOrDefault(i, null);
                            if (player != null) {
                                list.add(new Struct.PromotionData(player.getId(), player.data.avatarUrl, 0, 0, list.size() <= cnt));
                            }
                        }
                    }
                    break;
            }
            roomMap.put(room.roomId, list);
        }
    }

    /**
     * 比赛结算
     */
    private void settlement() {
        // 计算排行榜

        // 解散房间

        //
    }

    /**
     * 比赛结束
     */
    public void gameOver() {
        this.state = 3;
        this.time = 60;
        this.flag = false;
        Log.debug("比赛本轮结束...比赛 : {}, {}, 人数 : {}, 当前轮次 : {}",
                this.data.contestId, this.prop.desc, this.actors.size(), this.data.curRound);
    }

    /**
     * 比赛重置逻辑
     */
    public void resetContest() {

    }

    /**
     * 重置比赛，用于对象池回收前清理状态
     */
    public void reset() {
        this.isActive = false;
    }

    /**
     * 销毁比赛，释放资源
     */
    public void destroy() {
        this.reset();
        this.scheduler.shutdownNow();
        this.scheduler = null;
        // 从缓存中移除
        ContestService.getInstance().contestMap.remove(this.data.contestId);
        // 重置数据
        this.resetContest();
        // 归还对象
        ContestService.getInstance().contestPool.releaseContest(this);
        Log.debug("Contest {} has been destroyed.", this.data.contestId);
    }
}
