package com.incubator.game.contest;

import com.incubator.common.log4j.Log;
import com.incubator.common.util.RandomUtil;
import com.incubator.game.data.data.GameData;
import com.incubator.game.data.data.ext.ContestExt;
import com.incubator.game.player.Player;

import java.util.Map;
import java.util.concurrent.*;

/**
 * 官方比赛 管理类
 */
public class ContestService {

    private static ContestService instance;
    /** 比赛对象池 **/
    public ContestPool contestPool;
    /** 比赛缓存 **/
    public Map<Long, Contest> contestMap = new ConcurrentHashMap<>();

    public synchronized static ContestService getInstance() {
        if (instance == null) {
            instance = new ContestService();
        }
        return instance;
    }

    /**
     * 初始化
     *
     * @param maxIdleContest 最大空闲数量
     */
    public void init(int maxIdleContest) {
        Log.info("初始化官方比赛服务...");
        // 初始化对象池
        this.contestPool = new ContestPool(maxIdleContest);
        // 初始化策划表赛事
        for (ContestExt ext : GameData.Contests.values()) {
            if (ext != null && ext.isOpen == 1) {
                long contestId = RandomUtil.randomSevenDigits();
                this.creatContest(ext.gameType, contestId, ext);
                Log.info("初始化比赛 : id={}, desc={}, 开始时间={}", contestId, ext.desc, ext.openTime);
                return;
            }
        }
    }

    /**
     * 获取玩家报名的比赛
     *
     * @param playerId
     * @return
     */
    public Contest getContestByPlayerId(String playerId) {
        for (Contest contest : this.contestMap.values()) {
            if (contest != null && contest.actors != null && contest.actors.containsKey(playerId)) {
                return contest;
            }
        }
        return null;
    }

    /**
     * 创建比赛
     *
     * @param roomType  比赛房间玩法类型
     * @param contestId 比赛id
     * @param prop      比赛策划表配置
     * @return
     */
    public synchronized Contest creatContest(int roomType, long contestId, ContestExt prop) {
        // 从对象池获取比赛实例
        Contest contest = this.contestPool.acquireContest(roomType);
        // 初始化比赛数据
        contest.init(roomType, contestId, prop);
        // 开始主线程逻辑
        contest.startContestTask();
        // 加入缓存
        this.contestMap.put(contest.data.contestId, contest);
        return contest;
    }

    /**
     * 报名加入比赛
     *
     * @param player
     * @param contestId
     */
    public synchronized Contest joinContest(Player player, long contestId) {
        Contest contest = this.contestMap.getOrDefault(contestId, null);
        if (contest == null) {
            return null;
        }
        // 执行玩家加入比赛逻辑
        contest.joinContest(player);
        // 绑定官方比赛
        player.data.contestId = contestId;
        return contest;
    }
}
